#!/usr/bin/env python
# Copyright (c) 2005-2011 Userful Corporation.  All rights reserved.
""" Read generic config file, then create bash compatible config file.
This program reads a config file containing distro specific configuration, then
create a new config file that bash scripts can source during UMx startup.
Called From: dm-initialize.sh
Calling:
Parameters: osname, osver
Copyright (c) 2005-2010 Userful Corporation.  All rights reserved.
Change Log:
"""
import sys, getopt, os, pwd, time
import ConfigParser
common_section = "common"
def usage():
    print '''
<Usage>: umx-distro-config.py <options>
             -h, --help: This messae.
             -v, --verbose: Verbose Mode
             --osname: OS name
             --osver: OS ver
             --in: input config (generic config format)
             --out: output config (bash config format)'''
def create_os_specific_config(os_name, os_ver, in_file, out_file, verbose):
    '''Get OS specific configuration from in_file (e.g: /et/userful/umx-distro.conf),
    then create out_file (e.g: /tmp/distro.conf) that can be sourced in the shell scripts.
    '''
    config_file = ConfigParser.ConfigParser()
    config_file.optionxform = str
    try:
        config_file.read(in_file)
    except:
        print "Invalid config file, %s" % in_file
        sys.exit()
    if verbose:
        print "Parse %s" % in_file
    section_str = "%s %s" % (os_name, os_ver)
    if verbose:
        print "Distro specific section: %s" % section_str
    if not config_file.has_section(common_section) and \
       not config_file.has_section(section_str):
        print "Both [common] and [%s] sections do not exist, so skip it and use default." \
              % section_str
        sys.exit()
    try:
        f = open(out_file, "w")
    except:
        print "Can't create %s" % out_file
        sys.exit()
    for section_name in [common_section, section_str]:
        if config_file.has_section(section_name):
            section_items = config_file.items(section_name)
            for option, value in section_items:
                if verbose:
                    print "%s=\"%s\"" % (option,value)
                f.write("%s=\"%s\"\n" % (option,value))
    f.close()
def main():
    "Main function"
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hv", ["help", "verbose", "osname=", "osver=", "in=", "out="])
    except getopt.GetoptError, err:
        print str(err)
        usage()
        sys.exit(2)
    verbose = False
    os_name = None
    os_ver = None
    in_file = None
    out_file = None
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
            sys.exit()
        elif o in ("-v", "--verbose"):
            verbose = True
        elif o == "--osname":
            os_name = a.lower()
        elif o == "--osver":
            os_ver = a.lower()
        elif o == "--in":
            in_file = a
        elif o == "--out":
            out_file = a
    if not os_name or not os_ver:
        usage()
        sys.exit(2)
    if verbose:
        print "OSNAME=%s, OSVER=%s" % (os_name, os_ver)
    create_os_specific_config(os_name, os_ver, in_file, out_file, verbose)
if __name__ == "__main__":
    main()
