#!/bin/bash
# Copyright (c) 2005-2011 Userful Corporation.  All rights reserved.
#
#
#
#
#
#
logfile="/var/log/userful-start.log"
ETC_PATH="/etc/userful"
ARG=$1
chmod a+rw $logfile
function bin() {
echo $(cd `dirname $0` && pwd)	
}
function restore_pam() {
if [ -z "$DMPAM" ]; then
DMPAM=/etc/pam.d/gdm
fi
for P in `basename $DMPAM` xdm kde kdm; do
mv -f /etc/pam.d/$P.multi-save /etc/pam.d/$P
done
}
function restore_library() {
[[ -d /etc/userful/backup ]] || mkdir -p /etc/userful/backup
libGLpath=$(check_libGL_path restore)
if [ -n "$LIBGL" -a -L $libGLpath/$LIBGL ]; then
rm -f $libGLpath/$LIBGL
fi
LIB_NAME=`ls /etc/userful/backup/libGL.so.*.system 2> /dev/null`
if [ -n "$LIB_NAME" ]; then
LIB_NAME=`basename $LIB_NAME`
LIB_NAME=${LIB_NAME//\.system}  # strip .system suffix.
cp -f /etc/userful/backup/$LIB_NAME.system  $libGLpath/$LIB_NAME
fi
libdrmintel_path=$(check_libdrmintel_path restore)
if [ -n "$LIBDRM_INTEL" -a -L $libdrmintel_path/$LIBDRM_INTEL ]; then
rm -f $libdrmintel_path/$LIBDRM_INTEL
fi
LIB_NAME=`ls /etc/userful/backup/libdrm_intel.so.*.system 2> /dev/null`
if [ -n "$LIB_NAME" ]; then
LIB_NAME=`basename $LIB_NAME`
LIB_NAME=${LIB_NAME//\.system}  # strip .system suffix.
cp -f /etc/userful/backup/$LIB_NAME.system  $libdrmintel_path/$LIB_NAME
fi
if [ -n "$LIBGL" -a -e /usr/local/lib/$LIBGL ]; then
mv -f /usr/local/lib/$LIBGL /etc/userful/backup/
fi
if [ -n "$LIBDRM_INTEL" -a -e /usr/local/lib/$LIBDRM_INTEL ]; then
mv -f /usr/local/lib/$LIBDRM_INTEL /etc/userful/backup/
fi
ldconfig
}
function fix_gdm_local_display() {
if [ ! -h /usr/bin/Xorg ]; then
mv -f /usr/bin/Xorg /usr/bin/Xorg.umx-save
ln -sf /opt/userful/bin/login-server /usr/bin/Xorg
fi
}
function fix_mount() {
if [ ! -h /bin/mount ]; then
mv -f /bin/mount /bin/mount.umx-save
ln -sf /opt/userful/bin/userful-mount /bin/mount
fi
}
function restore_system()
{
if [ -e $ETC_PATH/os-info ]; then
source $ETC_PATH/os-info
fi
if [ -z "$OSNAME" -o -z "$OSVER" ]; then
log "OSNAME/OSVER is empty. Can't restore system.."
return 1
fi
log "Restore system for $OSNAME/$OSVER.."
gen_distro_conf $OSNAME $OSVER
if [ -f /tmp/distro.conf ]; then
source /tmp/distro.conf
fi
if [ -n "$DMCONFIG" -a -e $DMCONFIG ]; then
DMCONFIG_NAME=`basename $DMCONFIG`
if [ -e /etc/userful/setup/$DMCONFIG_NAME.$OSNAME-$OSVER ]; then
log "Remove $DMCONFIG"
rm -f $DMCONFIG $DMCONFIG.multi-save
fi
fi
[[ -n "$PKLA_CONF" ]] && rm -f $PKLA_CONF
}
function sanity_check()
{
log "Sanity Check.."
if [ ! -e $ETC_PATH/os-info ]; then
log "$ETC_PATH/os-info doesn't exist. Restore system."
/opt/userful/bin/dm-check-os.sh
restore_system
return 1
fi
source $ETC_PATH/os-info
if [ -z "$OSNAME" -o -z "$OSVER" ]; then
log "OSNAME/OSVER is empty. Restore system."
/opt/userful/bin/dm-check-os.sh
restore_system
return 1
else
OLD_OSNAME=$OSNAME; OSNAME=""
OLD_OSVER=$OSVER; OSVER=""
fi
/opt/userful/bin/dm-check-os.sh
if [ -e $ETC_PATH/os-info ]; then
source $ETC_PATH/os-info
fi
if [ "$OLD_OSNAME" != "$OSNAME" -o "$OLD_OSVER" != "$OSVER" ]; then
log "$OLD_OSNAME/$OLD_OSVER has changed to $OSNAME/$OSVER . Restore system."
restore_system
fi
}
source "$(bin)/gconf.sh"
source "$(bin)/userful-bash-lib.sh"
sanity_check
if [ -f /etc/userful/os-info ]; then
source /etc/userful/os-info
fi
if [ -z "$OSNAME" -o -z "$OSVER" ]; then
log "OSNAME/OSVER is not avaiable."
exit 1
fi
gen_distro_conf $OSNAME $OSVER
if [ -f /tmp/distro.conf ]; then
source /tmp/distro.conf
fi
if [ -e /etc/sysconfig/xtype ]; then
source /etc/sysconfig/xtype
else
log "/etc/sysconfig/xtype is not avaiable."
fi
if [ -z "$XTYPE" ]; then
XTYPE="MULTI"
fi
log "started..."
if [ "$OSNAME" == "ubuntu" -a "$OSVER" == "9.04" ]; then
if [ ! -e /etc/X11/umx-start ]; then
log "Create /etc/X11/umx-start."
ln -s /opt/userful/bin/umx-start /etc/X11/umx-start
fi
if [ -e /etc/init.d/gdm ]; then
log "Rename /etc/init.d/gdm"
mv -f /etc/init.d/gdm /etc/init.d/gdm.multi-save
update-rc.d -f gdm remove
fi
if [ -e /etc/init.d/kdm ]; then
log "Rename /etc/init.d/kdm"
mv -f /etc/init.d/kdm /etc/init.d/kdm.multi-save
update-rc.d -f kdm remove
fi
if [ ! -e /etc/event.d/dm-start ]; then
log "Add /etc/event.d/dm-start"
cp -f /etc/userful/setup/dm-start.upstart /etc/event.d/dm-start
fi
else
if [ "$XTYPE" == "SINGLE" -o "$XTYPE" == "single" ]; then
if [ "$ARG" == "runtime" -o "$ARG" == "install" ]; then
if [[ -f /etc/udev/rules.d/99-umx.rules ]]; then
rm -f /etc/udev/rules.d/99-umx.rules
fi
if [[  -f /etc/udev/rules.d/25-userful-blacklist.rules ]]; then
rm -f /etc/udev/rules.d/25-userful-blacklist.rules
fi
if [ -f /opt/kde3/share/config/kcmartsrc ]; then
rm -f /opt/kde3/share/config/kcmartsrc
fi
if [ -e /etc/modprobe.d/userful-kms.conf ]; then
mv -f /etc/modprobe.d/userful-kms.conf /etc/userful/
fi
exit 0
fi
if [ -n "$DMCONFIG" ]; then
if [ -e $DMCONFIG.multi-save ]; then
mv -f $DMCONFIG.multi-save  $DMCONFIG
elif [ -e $DMCONFIG.multi-remove ]; then
rm -f $DMCONFIG  $DMCONFIG.multi-remove
fi
fi
[[ -n "$PKLA_CONF" ]] && rm -f $PKLA_CONF
if [ -e /etc/gdm/PreSession/Default.multi-save ]; then
mv /etc/gdm/PreSession/Default.multi-save /etc/gdm/PreSession/Default 2> /dev/null
fi
if [ /etc/init.d/pulseaudio ]; then
if which chkconfig > /dev/null ; then
chkconfig --add pulseaudio > /dev/null
elif which update-rc.d > /dev/null ; then
update-rc.d pulseaudio defaults $PASEQUENCE > /dev/null
fi
fi
if [ "$STARTUP" == "upstart" ]; then
if [ -n "$UMXSTART" -a -e $UMXSTART ]; then
rm -f $UMXSTART
fi
fi
if [ -h /usr/bin/Xorg ]; then
rm -f /usr/bin/Xorg
mv -f /usr/bin/Xorg.umx-save /usr/bin/Xorg
fi
log "Restoring libGL and libdrm_intel"
restore_library
log "Restoring pam.d"
restore_pam
if [ -f /usr/share/gdm/autostart/LoginWindow/gnome-mag.desktop.multi-save ]; then
mv /usr/share/gdm/autostart/LoginWindow/gnome-mag.desktop.multi-save \
/usr/share/gdm/autostart/LoginWindow/gnome-mag.desktop
fi
if [ -f /etc/xdg/autostart/gnome-at-session.desktop.multi-save ]; then
mv /etc/xdg/autostart/gnome-at-session.desktop.multi-save \
/etc/xdg/autostart/gnome-at-session.desktop
fi
if [ -f /etc/xdg/autostart/pulseaudio.desktop.multi-save ]; then
mv /etc/xdg/autostart/pulseaudio.desktop.multi-save \
/etc/xdg/autostart/pulseaudio.desktop
fi
if [ -n "$DMSTART" -a "$DMSTART" == "disable" ]; then
if [ -n "$DMINIT" -a -e /etc/init.d/$DMINIT ]; then
if which chkconfig > /dev/null ; then
chkconfig --add $DMINIT > /dev/null
elif which update-rc.d > /dev/null ; then
update-rc.d $DMINIT defaults $DMSEQUENCE > /dev/null
fi
fi
if [ -n "$DMJOB" ]; then
DMJOBBACKUP=/etc/userful/$(basename $DMJOB).multi-save
if [ -e $DMJOBBACKUP ]; then
mv -f $DMJOBBACKUP $DMJOB 
fi
JOBNAME=`basename $DMJOB`
JOBNAME=${JOBNAME//\.conf}
initctl start $JOBNAME
fi
fi
elif [ "$XTYPE" == "MULTI" -o "$XTYPE" == "multi" ]; then
if [ "$ARG" == "runtime" -o "$ARG" == "install" ]; then
if [[ ! -f /etc/udev/rules.d/99-umx.rules ]]; then
cp -f /etc/userful/setup/99-umx.rules /etc/udev/rules.d/
fi
if [[ ! -f /etc/udev/rules.d/25-userful-blacklist.rules ]]; then
cp -f /etc/userful/setup/25-userful-blacklist.rules /etc/udev/rules.d/
fi
if [ -n "$DMJOB" -a -e $DMJOB ]; then
DMJOBBACKUP=/etc/userful/$(basename $DMJOB).multi-save
mv -f $DMJOB $DMJOBBACKUP
fi
fix_gdm_local_display
if [ -e /etc/userful/userful-kms.conf ]; then
cp -f /etc/userful/userful-kms.conf /etc/modprobe.d/
fi
exit 0
fi
if [ -n "$LIBGL" -a ! -e /usr/local/lib/$LIBGL ]; then
if [ -e /etc/userful/backup/$LIBGL ]; then
cp -f /etc/userful/backup/$LIBGL  /usr/local/lib/
fi
fi
if [ -n "$LIBDRM_INTEL" -a ! -e /usr/local/lib/$LIBDRM_INTEL ]; then
if [ -e /etc/userful/backup/$LIBDRM_INTEL ]; then
cp -f /etc/userful/backup/$LIBDRM_INTEL /usr/local/lib/
fi
fi
if [ -n "$DMCONFIG" ]; then
DMCONFIG_NAME=`basename $DMCONFIG`
if [ -e $DMCONFIG ]; then
if [ ! -e $DMCONFIG.multi-save -a ! -e $DMCONFIG.multi-remove ]; then
if [ -e $ETC_PATH/setup/$DMCONFIG_NAME.$OSNAME-$OSVER ]; then
log "Unable to find the backup file of $DMCONFIG. System might be updated.."
mv -f $DMCONFIG{,.umx-bak}
cp -f $ETC_PATH/setup/$DMCONFIG_NAME.$OSNAME-$OSVER $DMCONFIG
cp -f $DMCONFIG{,.multi-remove}
else
cp -f $DMCONFIG{,.multi-save}
fi
fi
else
if [ -e $ETC_PATH/setup/$DMCONFIG_NAME.$OSNAME-$OSVER ]; then
cp -f $ETC_PATH/setup/$DMCONFIG_NAME.$OSNAME-$OSVER $DMCONFIG
cp -f $DMCONFIG{,.multi-remove}
else
log "Unable to find $ETC_PATH/setup/$DMCONFIG_NAME.$OSNAME-$OSVER"
fi
fi
fi
if [ -n "$PKLA_CONF" ]; then
pkla_base=`basename $PKLA_CONF`

if [ -e $ETC_PATH/setup/$pkla_base.$OSNAME-$OSVER ]; then
cp -f $ETC_PATH/setup/$pkla_base.$OSNAME-$OSVER  $PKLA_CONF
fi
fi
if [ -e /etc/gdm/PreSession/Default ]; then
if ! grep umx /etc/gdm/PreSession/Default ; then
echo '/opt/userful/bin/umx-presession-init.sh \
--user=$USER --display=$DISPLAY' \
>> /etc/gdm/PreSession/Default
fi
fi

if [ -n "$DMSTART" -a "$DMSTART" == "disable" ]; then
if [ -n "$DMINIT" -a -e /etc/init.d/$DMINIT ]; then
if which chkconfig > /dev/null ; then
chkconfig --del $DMINIT > /dev/null
elif which update-rc.d > /dev/null ; then
update-rc.d -f $DMINIT remove > /dev/null
fi
fi
DMJOBBACKUP=/etc/userful/$(basename $DMJOB).multi-save
if [ -n "$DMJOB" -a -e $DMJOB ]; then
JOBNAME=`basename $DMJOB`
JOBNAME=${JOBNAME//\.conf}
initctl stop $JOBNAME
mv -f $DMJOB $DMJOBBACKUP
fi
fi
if [ "$STARTUP" == "upstart" ]; then
if [ -n "$UMXSTART" ]; then
if [ ! -e $UMXSTART ]; then
UMXSTART_NAME=`basename $UMXSTART`
cp -f /etc/userful/setup/$UMXSTART_NAME.$OSNAME-$OSVER $UMXSTART
fi
fi
fi
if [ /etc/init.d/pulseaudio ]; then
if which chkconfig > /dev/null ; then
chkconfig --del pulseaudio > /dev/null
elif which update-rc.d > /dev/null ; then
update-rc.d -f pulseaudio remove > /dev/null
fi
fi

if [ "$OSNAME" == "novell" -a "$OSVER" == "11" ]; then
sed -i -e "s/DISPLAYMANAGER_REMOTE_ACCESS=.*/DISPLAYMANAGER_REMOTE_ACCESS=\"yes\"/g" $DMCONFIG
sed -i -e "s/DISPLAYMANAGER_STARTS_XSERVER=.*/DISPLAYMANAGER_STARTS_XSERVER=\"no\"/g" $DMCONFIG
elif [ -n "$RENAME_XORG" -a "$RENAME_XORG" == "enable" ]; then
log "Fixing gdm local display."
fix_gdm_local_display
fi
gen_gconf_conf $OSNAME $OSVER
log "Update GConf configuration (statrup)"
set_gconf direct
log "Update startup programs"
if [ -f /usr/share/gdm/autostart/LoginWindow/gnome-mag.desktop ]; then
mv -f /usr/share/gdm/autostart/LoginWindow/gnome-mag.desktop{,.multi-save}
fi
if [ -f /etc/xdg/autostart/gnome-at-session.desktop ]; then
mv -f /etc/xdg/autostart/gnome-at-session.desktop{,.multi-save}
fi
if [ -f /etc/xdg/autostart/pulseaudio.desktop ]; then
mv -f /etc/xdg/autostart/pulseaudio.desktop{,.multi-save}
fi
fi # SINGLE or MULTI
fi
log "done."
run_level=(`runlevel`)
if [ "$STARTUP" == "upstart" -a "$ARG" != "install" -a "${run_level[1]}" != "S" ]; then
if [ -z "$UMXSTART" ] ||
[ -n "$UMXSTART" -a ! -e $UMXSTART ]; then
log "Exec umx-start from umx-init.sh..."
sleep 3
exec /opt/userful/bin/umx-start
fi
fi
