#!/usr/bin/perl
# Copyright (c) 2005-2011 Userful Corporation.  All rights reserved.
#
use strict;
use warnings;
use Getopt::Long;
my $verbose = '';
my $pcibus = '';
my $Driver = '';
my $phys = '';
my $output = '';
my %opt = ();
GetOptions ('phys=s', \$phys, 'driver=s', \$Driver, 'output=s', \$output );
if ( -e $output) {
    unlink $output;
}
if (my $log = probe_using_X($phys, $Driver)) {
    if (parse_X_log_edids($log)) {
        print "Parsing EDID succeed.\n";
    }
    else {
        print "{EE} Parsing EDID failed.\n";
    }
} 
else { # Faile to Probe EDID on the current video card.
    warn("{EE} X probing failed.\n");
}
sub probe_using_X {
    my ($phys, $Driver) = @_;
    my $tmp_conf = tmpfile();
    my $tmp_log = tmpfile();
    output($tmp_conf, <<EOF);
Section "Files"
    ModulePath      "/opt/MX11R7/lib/xorg/modules"
EndSection
Section "Device"
    Identifier "device"
    Driver "$Driver"
    Option "Device" "$phys"
EndSection
Section "Screen"
    Identifier "screen"
    Device "device"
    DefaultDepth 24
EndSection
Section "ServerLayout"
    Identifier "layout"
    Screen "screen"
EndSection
EOF
    my $ok = eval {
        local $SIG{ALRM} = sub { die "alarm\n" };
        alarm 10;
		open(STDERR, ">/dev/null");# redirect output of next command to /dev/null
		open(STDOUT, ">/dev/null");# redirect output of next command to /dev/null
        my $ok = 
           system("/opt/MX11R7/bin/Xorg :0 -ac -probeonly -logfile $tmp_log -config $tmp_conf") == 0;
        alarm 0;
        if ($ok) {
            return "true";
        }
        else {
            return;
        }
    };
    if ($@) {
        die unless $@ eq "alarm\n";
        print "{EE} X probing failed.\n";
        return;
    } else {
        if ($ok) {
	    my $log = cat_($tmp_log);
	    return $log;
        }
        else {
            return;
        }
    }
}
sub tmpfile() { 
    chomp(my $s = `mktemp /tmp/tmp.XXXXXXXXXX`); #- we could use simply mktemp with new mktemp
    eval "END { unlink '$s' }";
    $s;
}
sub output { my $f = shift; open(my $F, ">$f") or die "output in file $f failed: $!\n"; print $F $_ foreach @_; 1 }
sub cat_ { open(my $F, $_[0]) or return; my @l = <$F>; wantarray() ? @l : join '', @l }
sub parse_X_log {
    my ($log) = @_;
    parse_X_log_edids($log) and return 1;
    print "\n\n{DD} Ever reach here\n\n";
    0
      || $log =~ m!\bSize of device LFP \(local flat panel\) is (\d+) x (\d+)$!m 
      || $log =~ m!\b(\d+)x(\d+) panel \(ID \d+\) detected\.$!m
      || $log =~ m!\bPanel Size from BIOS: (\d+)x(\d+)$!m 
      || $log =~ m!\bPanel size is (\d+) x (\d+)$!m
      || $log =~ m!\b(\d+)x(\d+) \S+ LCD panel detected !m
      || $log =~ m!\bPanel is a (\d+)x(\d+) (?:color|monochrome) (?:TFT|dual scan) display$!m
      || $log =~ m!\bDetected panel size via BIOS: (\d+) x (\d+)$!m
      || $log =~ m! Panel Size = (\d+)x(\d+)$!m
      || $log =~ m!\b(?:TFT|DSTN|STN) Panel (\d+)x(\d+) found$!m
      || $log =~ m!\bSelected Panel Size is (\d+)x(\d+)$!m
      || $log =~ m!BIOS panel mode is bigger than probed programmed mode, continuing with BIOS mode.*?BIOS mode:.*?Modeline "(\d+)x(\d+)"!s
      or return;
    my ($X, $Y) = ($1, $2);
    print $opt{perl} ? "[ { preferred_resolution => { X => $X, Y => $Y } } ]" : "${X}x${Y}", "\n";
    1;
}
sub parse_X_log_edids {
    my ($log) = @_;
    my @edids;
    while ($log =~ /: EDID \(in hex\):\n((.*\n){8})/g) {
    
	my @lines = split '\n', $1;
        my $edid = join('', map { /:\s+([0-9a-f]{32})$/ && $1 } @lines);
	if (length $edid == 256) {
	    @edids = uniq(@edids, $edid);
	} else {
	    warn "bad EDID found\n";
	}
    }
    if (@edids) {
	print "[\n" if $opt{perl};
	foreach my $edid (@edids) {
	    open(my $F, '| ' . parse_edid());
	    print $F pack("C*", map { hex($_) } $edid =~ /(..)/g);
	    close $F;
	    print ",\n" if $opt{perl};
	}
	print "]\n" if $opt{perl};
    }
    @edids;
}
sub uniq { my %l; $l{$_} = 1 foreach @_; grep { delete $l{$_} } @_ }
sub parse_edid() {
	print "parse_edid\n";
    join(' ', '/opt/userful/bin/umx-parse-edid.pl --output', $output , '--id', $phys);
}
sub propagate_options {
    my (@l) = @_;
    map { $_ eq 'verbose' ? '-v' : "--$_" } grep { $opt{$_} } @l;
}
