#!/usr/bin/env python
# Copyright (c) 2005-2011 Userful Corporation.  All rights reserved.
import dbus
import dbus.service
if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
    import dbus.glib
import gobject
import subprocess
import os
import cups
from subprocess import Popen, PIPE
class ServerObject(dbus.service.Object):
    def __init__(self):
        bus_name = dbus.service.BusName('com.userful.Storage',bus=dbus.SystemBus())
        dbus.service.Object.__init__(self, bus_name, '/com/userful/Storage')
        self.called_time = 0
    def got_called(self):
        self.called_time = gobject.get_current_time()
    @dbus.service.method('com.userful.Storage')
    def EchoString(self, original):
        self.got_called()
        return original
    @dbus.service.method('com.userful.Storage', in_signature='sssss', out_signature='b')
    def Mount(self, uid, gid, volume, mountpoint, fstype):
        if fstype[:3] == "ext":
            cmd = 'mount %s %s' % (volume, mountpoint)
            if not self.__run_helper(cmd):
                return False
            cmd = 'chmod a+w %s' % mountpoint
            return self.__run_helper(cmd)
        else:
            cmd = 'mount -o owner,user,uid=%s,gid=%s %s %s' \
                  % (uid, gid, volume, mountpoint)
            return self.__run_helper(cmd)
    @dbus.service.method('com.userful.Storage', in_signature='s', out_signature='b')
    def Unmount(self, volume):
        cmd = 'umount %s' % volume
        return self.__run_helper(cmd)
    @dbus.service.method('com.userful.Storage', in_signature='s', out_signature='s')
    def GetAttribute(self, device):
        cmd = "blkid -s TYPE %s" % device
        return self.__run_helper(cmd, return_stdout=True)
    def timer_callback(self, loop):
        now_time = gobject.get_current_time()
        if now_time - self.called_time > 90:
            loop.quit()
        return True
    def __run_helper(self, cmd, return_stdout=False):
        '''
        Run helper program
        '''
        result = True
        proc = Popen(cmd, stdout = PIPE, stderr = PIPE, shell = True)
        stdout, stderr = proc.communicate()
        if proc.returncode != 0:
            result = False
        if return_stdout:
            if result:
                result = stdout
            else:
                result = ''
        return result
server = ServerObject()
loop = gobject.MainLoop()
gobject.timeout_add(180000, server.timer_callback, loop)
loop.run()
