#!/usr/bin/env python

# Copyright (c) 2005-2010 Userful Corporation.  All rights reserved.
# http://userful.com/

# control-panel.py

import gtk 
import gtk.glade
import os, sys
import time
import errno

import gettext

USERFUL_PYLIB_PATH="/opt/userful/pylib"
sys.path.append(USERFUL_PYLIB_PATH)
import ShellConfigParser
UMX_RELEASE_FILE="/etc/umx-release"

gtk.glade.bindtextdomain("userful-control-panel", "/usr/share/locale")
gtk.glade.textdomain("userful-control-panel")

gettext.bindtextdomain('userful-control-panel')
gettext.textdomain('userful-control-panel')

#gettext.bind_textdomain_codeset("userful-control-panel", "UTF-8")
#gettext.bind_textdomain_codeset("userful-control-panel", "ISO-8859-1")

_ = gettext.gettext

class PyGlade:

    def __init__(self):
        self.glade_filename = 'control-panel.glade'
        self.paths = []
        self.paths.append('./')
        self.paths.append('/usr/share/umx-utils/')
        self.debug = False
        
        self.config_changed = False
        return

    def get_system_mac_addr(self, device = "eth0"):
        saved_lang = os.environ["LANG"]
        os.environ["LANG"] = "C"
        if device == '':
            output = os.popen('/sbin/ifconfig -a')
        else:
            output = os.popen('/sbin/ifconfig -a ' + device)
            time.sleep(0.1)
        os.environ["LANG"] = saved_lang
        system_text=output.read()
        output.close()
        system_text = system_text.split(" ")
        get_next = False
        for item in system_text:
            if get_next:
                return item
            if item == "HWaddr":
                get_next = True
        return "unknown"

    def get_sysid(self):
        output, the_input, more_output = os.popen3("/opt/userful/bin/dm-gen-info -encode " + self.get_system_mac_addr(device="eth0"))
        sysid = ""
        #sysid =  output.read().strip()
        
        sysid = sysid + more_output.read().strip()
        output.close()
        return "E00-" + sysid
    #"E00-0000-0000-0000"

    def setup(self):
        if self.debug:
            sys.stderr.write("Setup...\n")

        glade_file = self.find_file(self.glade_filename)

        if os.access(glade_file, os.R_OK) == 1:
            self.gtk_glade_xml = gtk.glade.XML(glade_file)
        else:
            sys.exit(1)

        widget = self.gtk_glade_xml.get_widget("main_dialog")
        widget.set_icon_from_file(self.find_file("userful-logo.svg"))

        widget = self.gtk_glade_xml.get_widget("logo")
        widget.set_from_file(self.find_file("userful-logo-transparent-background.png"))


        # build up the text for the stuff to the right of the logo

        # software version
        cfg = ShellConfigParser.ShellConfigParser()

        try:
            cfg.read(UMX_RELEASE_FILE)
        except:
            version_text = "unknown or not installed"
        else:
            version_text = cfg.get("shell", "VERSION")
            
        version_label_text = _("<b>Userful Multiplier Version:</b> ") + version_text.strip('"')

        # os version
        issue_file = file('/etc/issue', 'r')
        for line in issue_file.readlines():
            if line.strip() == "":
                continue
            break
        issue_file.close()
        
        if line.upper().find("WELCOME") == 0:
            line = line[line.find(" ")+1:]
        if line.upper().find("TO") == 0:
            line = line[line.find(" ")+1:]

        index = line.upper().rfind("KERNEL")
        if index >= 0:
            line = line[:index]
        
        line = line.replace("\k", "")
        line = line.replace("\l", "")
        line = line.replace("\m", "")
        line = line.replace("\n", "")
        line = line.replace("\r", "")
        line = line.replace("-", "")

        line = line.strip()

        os_label_text = _("<b>OS Version:</b> ") + line

        # mac adddr
        mac = self.get_system_mac_addr()
        mac_addr_text = "<b>MAC Address:</b> " + mac

        # sysid
        sysid = self.get_sysid()
        sysid_text = "<b>SysID:</b> " + sysid

        stuff_text = version_label_text + "\n" + \
            os_label_text + "\n" + \
            mac_addr_text  + "\n" + \
            sysid_text + "\n"

        # set the actual text
        widget = self.gtk_glade_xml.get_widget("version_label")
        widget.set_use_markup(True)
        widget.set_markup(stuff_text)

        # check the box is we are in "Multi" mode
        checkmultiply = self.gtk_glade_xml.get_widget("checkbutton_multiply")
        checkmultiply.set_active(True)
        try:
            conf_file = file("/etc/sysconfig/xtype", "r")
        except IOError:
            checkmultiply.set_active(False)
            checkmultiply.set_sensitive(False)

        if checkmultiply.get_active():
            conf_file_cont = conf_file.readline()
            if conf_file_cont.strip() == "XTYPE=MULTI":
                checkmultiply.set_active(True)
            else:
                checkmultiply.set_active(False)
        
        # check to see if this is a UD - if so, disable using the install license button
        if os.access("/etc/init.d/Userful-Desktop", os.R_OK) == 1:
            widget = self.gtk_glade_xml.get_widget("register_button")
            widget.set_sensitive(False)
            checkmultiply.set_sensitive(False)
            widget = self.gtk_glade_xml.get_widget("install_license_label")
            widget.set_text(_("Userful Desktop licenses are controlled by Userful Manager"))
            widget.set_sensitive(False)
            widget = self.gtk_glade_xml.get_widget("multiply_check_label")
            widget.set_sensitive(False)

        # check if the multiuser device thingy exists, hide the button
        # to lanuch it if it's not
        if not os.access("/usr/bin/userful-device-assignment", os.R_OK) == 1:
            widget = self.gtk_glade_xml.get_widget("device_config_button")
            widget.hide()
            widget = self.gtk_glade_xml.get_widget("device_config_label")
            widget.hide()
            widget = self.gtk_glade_xml.get_widget("device_config_hseparator")
            widget.hide()

        # get a dictionary of all the class' methods
        signal_handlers = {}
        for key in dir(self.__class__):
            signal_handlers[key] = getattr(self, key)

        # this will connect all the signals defined in the glade file to functions
        # witth the same name, this will warn if signals are not handled 
        self.gtk_glade_xml.signal_autoconnect(signal_handlers)

    def set_debug(self, d):
        self.debug = d
        return

    def find_file (self, file_name):
        for path in self.paths:
            file_path = os.path.join(path, file_name)
            if os.path.isfile(file_path):
                break
        return file_path

        widget = self.gtk_glade_xml.get_widget("error_text_label")
        widget.set_text("Error: " + text)
        widget = self.gtk_glade_xml.get_widget("error_dialog")
        widget.show_all()
        return
    def hide_error(self):
        widget = self.gtk_glade_xml.get_widget("main_dialog")
        widget.set_sensitive(True)
    def get_system_mac(self):
        output = os.popen('/sbin/ifconfig -a eth0')
        time.sleep(0.1)
        system_text=output.read()
        output.close()
        system_text = system_text.split(" ")
        get_next = False
        for item in system_text:
            if get_next:
                return item
            if item == "HWaddr":
                get_next = True
        return 0

    ################ CALLBACKS ################

    def on_checkbutton_multiply_toggled(self, widget):
        self.config_changed = True
        
    def on_ok_button_clicked(self, widget):
        if self.config_changed:
            checkmultiply = self.gtk_glade_xml.get_widget("checkbutton_multiply")

            xtype_file_name = "/etc/sysconfig/xtype"
            if os.access(xtype_file_name, os.W_OK):
                conf_file = file(xtype_file_name, "w")

                if checkmultiply.get_active():
                    conf_file.write("XTYPE=MULTI\n")
                else:
                    conf_file.write("XTYPE=SINGLE\n")
                conf_file.close()

                # Update UMx setup based on the xtype.
                os.system("/opt/userful/bin/umx-init.sh runtime")
                
            else:
                err_dlg = self.find_file("error-dialog.py")
                os.system(err_dlg + " " + "\"" + _("Could not write configuration. Could not write to /etc/sysconfig/xtype.") + "\" &")
        gtk.main_quit()
        return 

    def on_register_button_clicked(self, widget):
        ret = os.system(self.find_file("register.sh") + " &");
        if ret == 1:
            err_dlg = self.find_file("error-dialog.py")
            message = _("Could not launch web browser. Visit http://keys.userful.com/.")
            os.system(err_dlg + " " + "\"" + message + "\" &")
        return

    def on_install_license_button_clicked(self, widget):
        os.system(self.find_file("check-and-install-license.sh") + " &");
        return

    def on_Help_OK_button_clicked(self, widget):
        gtk.main_quit()
        return 

    def on_readme_button_clicked(self, widget):
        # Changed Jul 16, 2009, old link: http://support.userful.com/wiki/index.php/Readme/Desktop_Multiplier
        url = "http://support.userful.com/wiki/index.php/Manuals/UMx"
        ret = os.system(self.find_file("launch-preferred-browser.sh") + " " + url + " &");

        if not ret == 0:
            os.system(self.find_file("help-dialog.py") + " &");

        return

    def on_report_button_clicked(self, widget):
        # url = "http://userful.com/support/issue?p=dm"
        ret = os.system(self.find_file("submit-debug-info.sh") + " -g &");
        #if ret == 1:
        #    err_dlg = self.find_file("error-dialog.py")
        #    message = "Could not launch web browser. Goto " + url + " in a web browser."
        #    os.system(err_dlg + " " + "\"" + message + "\" &")
        return

    def on_main_dialog_destroy(self, widget):
        gtk.main_quit()

    def on_device_config_button_clicked(self, widget):
        os.system("userful-device-assignment &");

    def on_website_button_clicked(self, widget):
        url = "http://userful.com/"
        ret = os.system(self.find_file("launch-preferred-browser.sh") + " " + url + " &");
        if not ret == 0:
            err_dlg = self.find_file("error-dialog.py")
            message = "Could not launch web browser. Goto " + url + " in a web browser."
            os.system(err_dlg + " " + "\"" + message + "\" &")
        return

    def on_license_details_button_clicked(self, widget):
        os.system(self.find_file("license-details.py &"))
        return
    def on_reassign_button_clicked(self, widget):
        (in_pipe, out_pipe) = os.popen4("/opt/userful/bin/usbinput-reconf.sh")
        #time.sleep(1)
        output = out_pipe.read()
        ret = out_pipe.close()
        in_pipe.close()
        #if not output == "":
        #    err_dlg = self.find_file("error-dialog.py")
        #    output = output.replace("`", "")
        #    output = output.replace("'", "")
        #    message = "Error: " + output
        #    os.system(err_dlg + " " + "\"" + message + "\" &")
        return
        
    def on_reconfig_button_clicked(self, widget):
        (in_pipe, out_pipe) = os.popen4("rm -f /etc/X11/userful.Mxorg.conf", mode='r')
        output = out_pipe.read()
        ret = out_pipe.close()
        in_pipe.close()
        err_dlg = self.find_file("error-dialog.py")
        if not output == "":
            output = output.replace("`", "")
            output = output.replace("'", "")
            message = "Error: Could not reset station settings. Reason: " + output
            os.system(err_dlg + " " + "\'" + message + "\' &")
        else:
            err_dlg += " type=sucess \"text=" + _("Successfully removed config file. System will reconfigure on next boot.") + "\" &"
            os.system(err_dlg)
        return

def main():
    debug = False
    if os.access("/opt/userful/debug", os.R_OK) == 1:
        debug = True

    PyGlade_ins = PyGlade()

    PyGlade_ins.set_debug(debug)

    PyGlade_ins.setup()

    gtk.main()

if __name__ == "__main__":
    main()

