#!/usr/bin/env python

# Copyright (c) 2005-2010 Userful Corporation.  All rights reserved.
# http://www.userful.com/

import gtk 
import gtk.glade
import os
import time
import sys

class ErrDlg:
    glade_filename = 'error-dialog.glade'
    paths = []
    paths.append('./')
    paths.append('/usr/share/umx-utils/')

    type = 'error'

    def __init__(self):
        glade_file = self.find_file(self.glade_filename)
        
        if os.access(glade_file, os.R_OK) == 1:
            self.gtk_glade_xml = gtk.glade.XML(glade_file)
        else:
            exit

        widget = self.gtk_glade_xml.get_widget("error_dialog")
        widget.set_icon_from_file(self.find_file("userful-logo.svg"))

        # get a dictionary of all the class' methods
        signal_handlers = {}
        for key in dir(self.__class__):
            signal_handlers[key] = getattr(self, key)

        # this will connect all the signals defined in the glade file to functions
        # witth the same name, this will warn if signals are not handled 
        self.gtk_glade_xml.signal_autoconnect(signal_handlers)

    def set_text (self, text):
        widget = self.gtk_glade_xml.get_widget("error_text_label")
        widget.set_text(text)
        widget = self.gtk_glade_xml.get_widget("error_dialog")
        widget.set_position(gtk.WIN_POS_CENTER_ALWAYS)
        return

    def set_type (self, type):
        if type != "error":
            widget = self.gtk_glade_xml.get_widget("error_image")
            if type == "sucess":
                widget.set_from_file(self.find_file("userful-logo.svg"))
        return

    def find_file (self, file_name):
        for path in self.paths:
            file_path = os.path.join(path, file_name)
            if os.path.isfile(file_path):
                break
        return file_path

    def show (self):
        widget = self.gtk_glade_xml.get_widget("error_dialog")
        widget.show()
        return
    def hide (self):
        widget = self.gtk_glade_xml.get_widget("error_dialog")
        widget.hide()
        return

    def set_end_action(self, a):
        self.action = a;
    def end_action(self):
        if self.action == "HIDE":
            self.hide()
            return
        elif self.action == "QUIT":
            gtk.main_quit()
            return

        # default to quit
        gtk.main_quit()
        

    ################ CALLBACKS ################

    def on_error_dialog_destroy(self, widget):
        self.end_action()
        return 

    def on_error_dialog_okbutton_clicked(self, error_dialog_ok_button):
        self.end_action()
        return 

def read_args():
   return_dict = {}
   for arg in sys.argv[1:]:
       try:
           (keyname, keyvalue)=arg.split('=',1)
       except:
           break
       keyvalue = keyvalue.strip()
       keyname = keyname.strip()
       return_dict[keyname]=keyvalue
   return return_dict


def main():

    errdlg_ins = ErrDlg()

    try:
        errdlg_ins.set_text(sys.argv[1])
    except:
        pass

    arg_dict = read_args()

    if arg_dict.has_key("text") and arg_dict["text"] != "":
        errdlg_ins.set_text(arg_dict["text"])

    if arg_dict.has_key("type") and arg_dict["type"] != "":
        errdlg_ins.set_type(arg_dict["type"])

    errdlg_ins.set_end_action("QUIT")

    errdlg_ins.show()

    gtk.main()

if __name__ == "__main__":
    main()

